package Data::Collection::Search;

#Author: Obiora Embry

use strict;
use warnings;
use XML::Bare;
use Fcntl ':flock'; #import LOCK_* constants
use Cwd;
use Data::Dumper;

require Exporter;
use vars qw(@EXPORT_OK @ISA);

@ISA = qw(Exporter Data::Collection);
@EXPORT_OK = qw(search elapsed_time show_search_help show_search_version);

our $VERSION = '0.2.1';

sub show_search_help
{

my ($self) = shift;

print qq{Usage (see below):\n\n};
print qq{user can search all removable media that have been added\n\n};
print qq{\texample \#1:\n};
print qq{perl find_files_cl.pl [-t search_term] [-c case_sens]\n\n}; 
print qq{perl find_files_cl.pl [--search_term search term] [--case_sens case sens]\n\n}; 
		
print qq{\t--help/-h: prints usage\n\n};
print qq{\t--version/-v: prints version\n\n};
print qq{\t--search_term <search term>/-t <search term>: the term that you want to find\n\n};
print qq{\t--case_sens <case sens>/-c <case sens>: specify if you want to have the search be case-sensitive [true or false]\n\n};

exit(0);

}

sub show_search_version
{

my ($self) = shift;

print qq{\nData::Collection::Search version $VERSION\n};

exit(0);

}

sub search
{

my ($self, $search, $case_sens) = @_;

my $cwd = cwd;

my @db = <$cwd/rem_disk_index/*.xml>; 

my ($xml, $root, $file, $index, @found, %saw) = ( '', '', '', '', (), () );

foreach ( @db )
{

	$_ =~ /rem_disk_index\/(.*?)_index\.xml/ig;
	$file = qq{$cwd/removable_disk/xml/$1.xml};

	eval { $xml = new XML::Bare(file => $_ ) };

	die qq{error: $@ } if ( $@ );
  
	$root = $xml->parse();

	foreach $index ( @{$root->{removable_disk}->{index}} )
	{

			push (@found, $file) if ( ( $index->{value} =~ /$$search/ && $$case_sens ) || (  $index->{value} =~ /$$search/i && !$$case_sens ) );

	}

}

@found = sort { $a cmp $b } @found;

my @found_up = grep ( !$saw{$_}++, @found );

$self->search_xml(\$cwd, $search, $case_sens, \@found_up);

}

sub search_xml
{

my ($self, $cwd, $search, $case_sens, $file) = @_;

my ($xml, $num, $root, $xml_file, $index, $file_name, $size, $mode, $last_modified, @file_name, @case_sens, @size, @mode, @last_modified, @disk) = ( '', 0, '', '', '', '', '', '', '', (), (), (), (), (), () );

my ($report_location, $semaphore) = $self->_create_report($cwd);

foreach ( @$file )
{

	$xml_file = $_;
	$xml_file =~ s!\Q$$cwd/removable_disk/xml/\E!!g;
	$xml_file =~ s!\.xml!!g;

	eval { $xml = new XML::Bare(file => $_ ) };

	die qq{error: $@ } if ( $@ );
  
	$root = $xml->parse();

	foreach $index ( @{$root->{removable_disk}->{'rd_'.$xml_file}->{file}} )
	{

		$file_name = ( $index->{file_name}->{value} ? $index->{file_name}->{value} : 'Unknown' ) if ( ( $index->{file_name}->{value} =~ /$$search/ && $$case_sens ) || ( $index->{file_name}->{value} =~ /$$search/i && !$$case_sens ) );
		$size = ( $index->{size}->{value} ? $index->{size}->{value} : 'Unknown' ) if ( ( $index->{file_name}->{value} =~ /$$search/ && $$case_sens ) || ( $index->{file_name}->{value} =~ /$$search/i && !$$case_sens ) );
		$mode = ( $index->{mode}->{value} ? $index->{mode}->{value} : 'Unknown' ) if ( ( $index->{file_name}->{value} =~ /$$search/ && $$case_sens ) || ( $index->{file_name}->{value} =~ /$$search/i && !$$case_sens ) );
		$last_modified = ( $index->{last_modified}->{value} ? $index->{last_modified}->{value} : 'Unknown' ) if ( ( $index->{file_name}->{value} =~ /$$search/ && $$case_sens ) || ( $index->{file_name}->{value} =~ /$$search/i && !$$case_sens ) );

		$self->_add_report(\$xml_file, \$file_name, \$size, \$mode, \$last_modified) if ( ( $index->{file_name}->{value} =~ /$$search/ && $$case_sens ) || ( $index->{file_name}->{value} =~ /$$search/i && !$$case_sens ) );

		$num++ if ( ( $index->{file_name}->{value} =~ /$$search/ && $$case_sens ) || ( $index->{file_name}->{value} =~ /$$search/i && !$$case_sens ) );
	
	}

	push (@disk, $xml_file);

}

$self->_close_report($semaphore);

my ($i, @disks) = ( 0, () );
map { $i++; $_ = ( $i == $#disk + 1 ? $_ : $_.',' ); push (@disks, $_); } @disk;

return ($report_location, \$num, \@disks);

}

sub _create_report
{

my ($self, $cwd) = @_;

Data::Collection->_mk_dir(\qq{$$cwd/DC_report});
my $file_name = qq{$$cwd/DC_report/DC_search_results.xml};

my @alphanumeric = ('a'..'z', 'A'..'Z', 0..9);
my $rand_alphanumeric = join ('', map $alphanumeric[rand @alphanumeric], 0..9);

my $SEMAPHORE = $file_name.$rand_alphanumeric.'.lck';
open(S, ">", "$SEMAPHORE") or die qq{Could not create $SEMAPHORE: $!\n};
flock(S, LOCK_EX) or die qq{flock() failed for $SEMAPHORE: $!\n};			
open(XML, ">", "$file_name") or die qq{Could not create file name $file_name\n};

print XML qq{<?xml version="1.0" encoding="UTF-8"?>\n\n};
print XML qq{<files_found>\n};

return (\$file_name, \$SEMAPHORE);

}

sub _add_report
{

my ($self, $disk_name, $file_name, $size, $mode, $last_modified) = @_;

print XML qq{<file>\n};
print XML qq{<disk_name>$$disk_name</disk_name>\n};
print XML qq{<file_name>$$file_name</file_name>\n};
print XML qq{<mode>$$mode</mode>\n};
print XML qq{<size>$$size</size>\n};
print XML qq{<last_modified>$$last_modified</last_modified>\n};
print XML qq{</file>\n};

return;

}

sub _close_report
{

my ($self, $SEMAPHORE) = @_;

print XML qq{</files_found>\n};

close(XML);
close(S);

Data::Collection->_delete_file($SEMAPHORE);

return;

}

sub elapsed_time
{

my ($self) = shift;

my $total_seconds = $_[2] - $_[0];
$total_seconds .= '.';
$total_seconds .= $_[3] - $_[1] if ( $_[3] > $_[1] );
$total_seconds .= $_[1] - $_[3] if ( $_[1] > $_[3] );

return \$total_seconds;

}

1;

__END__
